@import url('https://fonts.googleapis.com/css2?family=Roboto+Condensed:ital,wght@0,100..900;1,100..900&family=Roboto+Flex:opsz,wght,XOPQ,XTRA,YOPQ,YTDE,YTFI,YTLC,YTUC@8..144,100..1000,96,468,79,-203,738,514,712&family=Roboto:ital,wght@0,100..900;1,100..900&display=swap');

body {
    background-color: #ECEFF1;
    font-family: "Roboto", sans-serif;
    font-weight: 400;
    margin: 0px;
}

h1 {
    font-size: 1.4em;
}

h2 {
    font-size: 1.2em;
}

h3 {
    font-size: 1em;
}
    
h1, h2, h3 {
    font-family: "Roboto Condensed";
    font-weight: normal;
    color: #bf360c;
}

blockquote {
    font-style: italic;
}

a,
a:visited {
    color: #0288d1;
}

.link-button:visited,
.link-button {
    display: block;
    padding: 10px;
    border: 1px solid #ccc;
    border-radius: 4px;
    color: #fff;
    background-color: #00796b;    
    text-decoration: none;
    text-align: center;
    cursor: pointer;
}

.menu select,
#container select,
#container button,
#container input {
    font-family: "Roboto", sans-serif;
}

.menu select {
    font-size: 1em;
    background: none;
    border: none;
}

.menu select:focus-visible {
    outline: none;
}

.menu .tool-item {
    padding: 5px 0px;
}

.menu .model-buttons,
.menu .login-buttons {
    margin-bottom: 10px;
}

.model-buttons-wrapper {
  display: flex;
  flex-direction: column;
}

.model-buttons-row {
  display: flex;
  justify-content: center;
  gap: 10px;
  margin-bottom: 10px;
}

.model-buttons-row button {
  flex: 1;
}

.codehilite pre {
    overflow: auto;
    line-height: 150%;
}

#message-counter,
#footer-bar {
    color: gray;
    font-size: 0.7em;
    text-align: right;
    font-family: 'Roboto Condensed';
    margin-top: 8px;
    margin-bottom: 8px;
}

#footer-bar a {
    color: gray;
}

#sign-out {
    width: 100%;
    margin: auto;
    text-align: right;
    font-size: .6em;
    margin-bottom: 10px;
}

#header-bar {
    display: flex;
    justify-content: center;
    margin: 0 auto
}

#header-logo {
    max-width: 100%;
    height: auto;
    border-radius: 4px;
    border: solid 1px black;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
    margin-bottom: 10px;
}

#container select,
#container input[type="text"],
#container input[type="password"] {
    padding: 10px;
    margin-bottom: 10px;
    border: 1px solid #ccc;
    border-radius: 4px;
}

.message-info {
    margin-bottom: 10px;
    font-size: .8em;
    color: #607D8B;
}

#page-notification {
    border-radius: 4px;
    color: #fff;
    background-color: #00796b;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
    padding: 10px;
    margin-bottom: 10px;
    font-size: .8em;
}

#page-notification a {
    color: #fff;
}

.message-user {
    color: #00796b;
    white-space: pre-wrap;
}

.message-loading {
    color: gray;
}

.message-loading,
.message-user,
.message-ai {
    margin-bottom: 10px;
}

.codehilite pre {
    padding: 10px;
}

#message-box {
    display: flex;
    justify-content: space-between;
    margin-bottom: 10px;
    margin-top: 10px;
    min-height: 120px;
    position: sticky;
    bottom: 0;
    padding: 10px;
    background-color: #FFFDE7;
    border-radius: 4px;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
}

.expanded-message-box {
    height: 80vh;
}

#message-container {
    width: 100%;
    display: flex;
    align-items: stretch;
    padding: 10px;
    border: 1px solid #ccc;
    border-radius: 4px;
    background-color: white;
}

#button-container {
    display: flex;
    flex-direction: column;
    align-self: flex-start;
}

textarea#message {
    border: none;
    flex-grow: 1;
    height: 100%;
}

textarea#message:focus {
    outline: none;
}

textarea#message:disabled {
    background: none;
}

button#attach,
button#cancel,
button#expand,
button#send {
    font-size: 1.2em;
    height:100%;
    border: none;
    color: #fff;
    color: #00796b;
    background: none;
    cursor: pointer;
}

button#cancel {
    display: none;
}

button#send:disabled {
    color: #cccccc;
}

input#submit,
button#start {
    width: 100%;
    padding: 10px;
    border: none;
    color: #fff;
    background-color: #00796b;
    border-radius: 4px;
    cursor: pointer;
}

#response {
    margin-bottom: 10px; 
    word-wrap: break-word;
}

#start-text p,
#response p {
    margin: 0px;
    margin-block-end: 10px;
}

#start-text  ul {
    padding-left: 18px;
}

#loading {
    margin-bottom: 10px;
}


.form-group label {
    display: block;
    margin-bottom: 5px;
}

#start-info button {
    width: 100%;
}

#control-box {
    margin-bottom: 10px;
}

#control-box button {
    font-size: 0.6em;
    padding: 5px;
    border: none;
    color: black;
    background-color: #eeeeec;
    border-radius: 4px;
    cursor: pointer;
}

#control-box button:hover {
    text-decoration: underline;
}

#start-text {
    margin-top: 0px;
    padding: 10px 20px 10px 20px;
    border: 1px solid #ccc;
    border-radius: 4px;
    background-color: white;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
}

#library {
    margin-left: 10px;
    margin-right: 10px;
}

#example-queries {
    display: flex;
    flex-wrap: wrap;
    justify-content: space-between;
}

.example-query {
    flex: 1 0 30%;
    margin: 10px;
    font-size: 0.6em;
    padding: 10px;
    color: black;
    background: white;
    border: 1px solid #ccc;
    border-radius: 4px;
    cursor: pointer;
}

.example-query:hover {
    color: #fff;
    background-color: #00796b;
}

#user-message {
    border-top: solid 1px;
    padding-top: 10px;
    font-size: 0.6em;
    margin-bottom: -10px;
}

.thinking_block_signature {
    display: none;
}

.thinking_block_content {
    font-family: "Roboto Condensed";
    font-size: 0.6em;
    color: #90A4AE;
    padding-left: 10px;
    border-left: solid 1px #90A4AE;
    margin-bottom: 10px;
}

.message-workspace,
.message-timestamp,
.message-answer-model,
.message-sources {
    font-family: "Roboto Condensed";
    font-size: 0.6em;
    text-align: right;
    color: #90A4AE;
    white-space: normal;
}

.message-workspace .workspace-content,
.message-workspace .workspace-language {
    display: none;
}

.message-ai,
.message-user {
    border-radius: 4px;
    padding: 10px;
    box-shadow: 0 0 10px rgba(0,0,0,0.1);
}

.message-ai {
    margin-right: 10%;
    background-color: #ffffff;
}

.message-user {
    margin-left: 10%;
    background-color: #FFFDE7;
}

.attachment-item {
	font-family: monospace;
	color: gray;
}

.attachment-item button {
	background: none;
	border: none;
}

.hidden-data {
    display: none;
}

.message-delete {
    float: right;
    padding: 0px;
    border: none;
    background: none;
    display: none;
    font-size: 0.6em;
    color: #90A4AE;    
}

.message-user:hover .message-delete,
.message-ai:hover .message-delete {
    display: block;
}

.pretty-text {
    font-family: "Lobster Two";
    background-color: #ffd600;
    border-radius: 4px;
    padding: 8px;
    float: right;
    margin: 0px 0px 20px 20px!important;
}

.google-scholar-search-results,
.attachment-content {
    margin-bottom: 10px;
    border-radius: 4px;
    padding: 10px;
    max-height: 100px;
    font-size: 0.7em;
    overflow-y: auto;
    background-color: #cfd8dc;
}

.message .message-notification {
    border-top: solid 1px #90A4AE;
    padding-top: 10px;
    padding-bottom: 10px;
    color: #90A4AE;
    font-size: .8em;
}

.message .image-generation {
    margin: 10px 0px 10px 0px;
    border: solid 1px #90A4AE;
    border-radius: 4px;
    padding: 8px;
    background-color: white;
}

.message .image-generation img {
    width: 100%;
}

.login-text {
    text-align: center;
    margin-bottom: 30px;
}

.menu small a,
.menu small {
    font-family: "Roboto Condensed";
    color: #aaa;
}

#container {
    display: flex;
    flex-direction: column;
    height: 100%;
}

#centered-container {
    display: grid;
    justify-content: center;
    margin-top: 10px;
    width: 100%;
}

#centered-container #start-text {
    max-width: 558px;
}

#centered-container #header-bar {
    max-width: 598px;
}

#centered-container #login {
    max-width: 602px;
}

#workspace-placeholder {
    position: absolute;
    font-family: "Roboto Condensed";
    font-size: 1.2em;
    color: gray;
    text-align: center;
}

#connected-status {
    display: none;
    padding: 10px;
    text-align: center;
    font-family: "Roboto Condensed";
}

#code-editor {
    background-color: #f0f0f0;
    overflow: auto;
}

#code-editor-options {
    position: absolute;
    bottom: 0;
    right: 0;
    padding: 8px;
    z-index: 1;
}

#code-editor-options option {
    background-color: #272822;
    color: #f8f8f2;
}

#code-editor-options select,
#code-editor-options button {
    padding: 4px;
    background-color: transparent;
    border: none;
    color: #f8f8f2;
}

#chat-area {
    background-color: #e0e0e0;
    overflow: auto;
    overflow-x: hidden;
    padding: 10px 10px 0px 10px;
}

/* Large screens */
@media (min-width: 768px) {
    #container {
        flex-direction: row;
    }
    
    .CodeMirror {
        height: 100vh!important;
    }

    #code-editor {
        flex: 1;
        height: 100vh;
    }

    #chat-area {
        width: 600px;
        height: calc(100vh - 10px);
    }
    
    #workspace-placeholder {
        top: 10%;
        width: calc(-620px + 100vw);
    }    
}

/* Small screens */
@media (max-width: 767px) {
    #code-editor {
        height: 190px; /* Adjust as needed */
    }

    #chat-area {
        height: calc(100vh - 200px); /* Adjust as needed */
    }
    
    #workspace-placeholder {
        top: calc(100vh - 200px); /* Adjust as needed */
        margin: 50px;
    }
}

/* Connection Lost Modal */
.modal {
    display: none;
    position: fixed;
    z-index: 10000;
    left: 0;
    top: 0;
    width: 100%;
    height: 100%;
    background-color: rgba(0, 0, 0, 0.6);
    align-items: center;
    justify-content: center;
}

.modal-content {
    background-color: #fff;
    padding: 30px;
    border-radius: 8px;
    max-width: 500px;
    box-shadow: 0 4px 20px rgba(0, 0, 0, 0.3);
    animation: modalSlideIn 0.3s ease-out;
}

@keyframes modalSlideIn {
    from {
        transform: translateY(-50px);
        opacity: 0;
    }
    to {
        transform: translateY(0);
        opacity: 1;
    }
}

.modal-content h2 {
    margin-top: 0;
    color: #d32f2f;
    font-size: 24px;
}

.modal-content p {
    line-height: 1.6;
    color: #333;
}

.modal-content ul {
    margin: 15px 0;
    padding-left: 20px;
}

.modal-content li {
    margin: 8px 0;
    color: #555;
}

.modal-reload-button {
    background-color: #00796b;
    color: white;
    padding: 12px 30px;
    border: none;
    border-radius: 4px;
    cursor: pointer;
    font-size: 16px;
    font-weight: bold;
    margin-top: 20px;
    width: 100%;
    transition: background-color 0.3s;
}

.modal-reload-button:hover {
    background-color: black;
}

/* Error message styling */
.message-error {
    border-left: 4px solid #d32f2f;
    background-color: #ffebee;
}

.message-error strong {
    color: #d32f2f;
}

.why-sigmund {
    background-color: #00796b;
    color: white;
    padding: 20px!important;
    margin: 40px 10px 10px 10px;
    border-radius: 4px;    
}

.why-sigmund p {
    font-size: 1.2em;
    font-family: "Roboto Condensed";
    margin: 0px;
}

#usage-counter {
    position: relative;
    height: 1.4em;
    border-radius: 3px;
    overflow: hidden;
    border: solid 1px #ccc;
}

#usage-bar {
    position: absolute;
    top: 0;
    left: 0;
    height: 100%;
    transition: width 0.3s ease;
}

#usage-bar.usage-low {
    background: #00796b; /* green */
}

#usage-bar.usage-medium {
    background: #ff9800; /* orange */
}

#usage-bar.usage-high {
    background: #f44336; /* red */
}

#usage-label {
    font-family: "Roboto Condensed";
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
    font-size: 0.7em;
    white-space: nowrap;
    z-index: 10;
}

#usage-label.usage-low {
    color: gray;
}

#usage-label.usage-medium,
#usage-label.usage-high  {
    color: white;
}

#usage-limit-warning {
    background: #f44336;
    color: white;
    padding: 10px;
    border-radius: 4px;
    margin-bottom: 10px;
}

#usage-limit-warning a {
    color: white; 
}

#usage-limit-warning.hidden {
    display: none; 
}
