<!DOCTYPE html>
<html>
{% include 'head.html' %}
<body>
    <div id="centered-container">
        {% include 'header.html' %}
        <div id="start-text">
<p>
An internal server error occurred! :/
</p>

<p>
Please reload the page and try again. If the problem persists:
</p>
     
<ul> 
<li>  
    <a href="/api/conversation/new">Start a new conversation</a>
</li>
<li>  
    Or <a href="/app/logout">log out</a> 
</li>
<li>  
    Or <a href="mailto:professional@cogsci.nl">contact the system administrators</a>
</li>
</ul>
        </div>
        {% include 'footer.html' %}
    </div>
</body>
</html>
